﻿<?php


require_once 'lib/user_input_handler_registry.php';
require_once 'lib/utils.php';
require_once 'lib/action_factory.php';
require_once 'lib/control_factory.php';
require_once 'iptvportal_config.php';


class IptvportalInputHandler implements UserInputHandler {
    public function __construct () {
        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    public function get_handler_id () {
        return "main_handler";
    }

    public function handle_user_input (&$user_input, &$plugin_cookies) {
        hd_print('handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        if ($user_input->op == 'load_portal') {
            $res = IptvportalPlugin::login_terminal (null);
            if (!$res)
                return ActionFactory::show_dialog ('Введите регистрационные данные', $this->registration_form_defs ($plugin_cookies), true);

            IptvportalPlugin::run_arescrypt ($plugin_cookies);
            return ActionFactory::launch_media_url (IptvportalConfig::portal_url);
        } else if (isset ($user_input->control_id)) {
            $control_id = $user_input->control_id;
            if ($control_id == 'register_terminal') {
                if (is_null ($user_input->username) || trim ($user_input->username) === '') {
                    return ActionFactory::show_error (false, 'Ошибка', array ('Имя абонента не может быть пустым.'));
                } else if (is_null ($user_input->password) || trim ($user_input->password) === '') {
                    return ActionFactory::show_error (false, 'Ошибка', array ('Пароль не может быть пустым.'));
                }
                $res = IptvportalPlugin::register_terminal ($user_input->username, $user_input->password);
                if (!$res)
                    return ActionFactory::show_error (false, 'Ошибка', array ('Неправильное имя абонента или пароль.'));
                $plugin_cookies->username = $user_input->username;
                $plugin_cookies->password = $user_input->password;
    
                IptvportalPlugin::run_arescrypt ($plugin_cookies);
                return ActionFactory::close_dialog_and_run (ActionFactory::launch_media_url (IptvportalConfig::portal_url));
            } else if ($control_id == 'register_cancel') {
                return ActionFactory::close_dialog ();
                $res = IptvportalPlugin::login_terminal (null);
                if (!$res)
                    return ActionFactory::reset_controls ($this->registration_form_defs ($plugin_cookies));
    
                IptvportalPlugin::run_arescrypt ($plugin_cookies);
                return ActionFactory::close_dialog_and_run (ActionFactory::launch_media_url (IptvportalConfig::portal_url));
            }
        }

        return null;
    }

    private function registration_form_defs (&$plugin_cookies) {
        $defs = array ();

        $username = isset($plugin_cookies->username) ? $plugin_cookies->username : '';
        $password = isset($plugin_cookies->password) ? $plugin_cookies->password : '';
        $unregister = isset($plugin_cookies->unregister) ? $plugin_cookies->unregister : false;

        ControlFactory::add_text_field ($defs, $this, null, 'username', 'Абонент:', $username,
                                false/*numeric*/, false, false, false, 500);
        ControlFactory::add_text_field ($defs, $this, null, 'password', 'Пароль:', $password,
                                false/*numeric*/, true, false, false, 500);
        ControlFactory::add_vgap ($defs, 50);
        ControlFactory::add_button ($defs, $this, null, 'register_terminal', null, 'Регистрация', 400);
        //ControlFactory::add_vgap ($defs, -3);
        ControlFactory::add_button ($defs, $this, null, 'register_cancel', null, 'Отменить', 400);

        return $defs;
    }
}


class IptvportalPlugin implements DunePlugin {
    private $handler;
    private static $arescam_exe;

    public function __construct () {
        IptvportalPlugin::$arescam_exe = dirname(__FILE__) . '/bin/arescam';

        $handler = new IptvportalInputHandler ();
    }

    public function handle_user_input (&$user_input, &$plugin_cookies) {
        return UserInputHandlerRegistry::get_instance()->handle_user_input ($user_input, $plugin_cookies);
    }

    public function get_folder_view ($media_url_str, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function get_next_folder_view($media_url_str, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function get_regular_folder_items ($media_url_str, $from_ndx, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function get_tv_info ($media_url_str, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function get_tv_playback_url ($channel_id, $archive_ts, $protect_code, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function get_tv_stream_url ($playback_url, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function get_day_epg ($channel_id, $day_start_ts, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function change_tv_favorites ($fav_op_type, $channel_id, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function get_vod_info ($media_url_str, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public function get_vod_stream_url ($playback_url, &$plugin_cookies) {
        throw new Exception ("Not supported");
    }

    public static function login_terminal ($anonymous)
    {
        // NOTE: 'chmod +x arescam' is needed because zip does not preserve file
        // attributes. So, after unpacking 'arescam' is not executable.
        $cmd = sprintf('chmod +x %s', escapeshellcmd (IptvportalPlugin::$arescam_exe));
        hd_print ("chmod command: $cmd");
        $out = system ($cmd, $rc);
        if ($out === false) {
            hd_print ("Warning: 'chmod +x' failed for 'arescam'.");
        }
        if (!is_null ($anonymous)) {
            $cmd = sprintf (IptvportalConfig::cmd_login_anonymous,
                        escapeshellcmd (IptvportalPlugin::$arescam_exe), escapeshellarg (IptvportalConfig::auth_url), $anonymous);
        } else {
            $cmd = sprintf (IptvportalConfig::cmd_login,
                        escapeshellcmd (IptvportalPlugin::$arescam_exe), escapeshellarg (IptvportalConfig::auth_url));
        }
        hd_print ("cmd_login: $cmd");
        $out = system ($cmd, $rc);
        if ($out === false) {
            hd_print ("Error: can not execute 'arescam'.");
            throw_ares_error ();
        }
        $out = chop ($out);
        hd_print ("arescam -l: rc: $rc; output: $out");
        if ($rc === 0) {
            return true;
        }
        try {
            hd_print ('--> Checking server availability...');
            HD::http_get_document (IptvportalConfig::api_url);
            hd_print ('--> Server is available.');
        } catch (Exception $e) {
            throw_server_unavailable_error ();
        }
        hd_print ('--> Login failed.');
        return false;
    }

    public static function register_terminal ($username, $password) {
        $cmd = sprintf (IptvportalConfig::cmd_register,
            escapeshellcmd (IptvportalPlugin::$arescam_exe), escapeshellarg (IptvportalConfig::auth_url),
            escapeshellarg ($username), escapeshellarg ($password));
                
        hd_print ("cmd_register: $cmd");
        $out = system ($cmd, $rc);
        if ($out === false) {
            hd_print ("Error: can not execute 'arescam'.");
            throw_ares_error ();
        }
        $out = chop ($out);
        hd_print ("arescam -r: rc: $rc" . '( ' .  ($rc === 0 ? 'Success' : 'Failure') . '); ' . "output: $out");
        return ($rc === 0);
    }

    public static function run_arescrypt (&$plugin_cookies) {
        // Here we should insert arescrypt.ko and start arescam-daemon.
        //
        // The thing is that we might already have the daemon running and
        // the kernel module inserted (that might happen if PHP engine was
        // restarted in the middle).
        //
        // The logic will be as follows:
        //   - kill arescam (if it is running):
        //     - check PID file;
        //     - remove PID file;
        //   - remove arescrypt.ko;
        //   - detect smp family;
        //   - insert arescrypt.ko;
        //   - start up arescam-daemon;
        //   - check new PID file.

        // 1. Kill arescam.

        hd_print ("--> Reading PID-file '" . IptvportalConfig::daemon_pid_file . "'...");
        hd_silence_warnings ();
        $pid = file_get_contents (IptvportalConfig::daemon_pid_file);
        hd_restore_warnings ();
        if ($pid !== false) {
            hd_print ("--> PID: $pid");
            $cmd = sprintf ('kill %s', escapeshellarg ($pid));
            hd_print ("cmd_kill: $cmd");
            system ($cmd);
            hd_print ("--> unlinking " . IptvportalConfig::daemon_pid_file);
            unlink (IptvportalConfig::daemon_pid_file);
        }

        // 2. rmmod arescrypt.ko

        $cmd = sprintf ('rmmod arescrypt');
        hd_print("cmd_rmmod: $cmd");
        system ($cmd);

        // 3. detect smp family and arescrypt kernel module

        $kernel_release = system ('uname -r', $rc);
        if ($rc !== 0) {
            hd_print ("Error: uname -r failed");
            throw_ares_error ();
        }
        $cmd = sprintf (IptvportalConfig::cmd_smp_family);
        //hd_print ("cmd_smp_family: $cmd");
        $smp_family = system ($cmd, $rc);
        if ($smp_family === false) {
            hd_print ("Error: can not detect smp family.");
            throw_ares_error ();
        }
        $smp_family = strtolower (chop ($smp_family));
        hd_print ("smp family: $smp_family");
        $arescrypt_ko = dirname(__FILE__) . '/modules/' .
                        $kernel_release . '/' . $smp_family . '/arescrypt.ko';
        if ($smp_family == "867x") {
            $cmd = sprintf (IptvportalConfig::cmd_kernel_chksum);
            $kernel_chksum = system ($cmd, $rc);
            hd_print ("kernel config chksum: $kernel_chksum");
            $chksum_parts = explode (" ", $kernel_chksum);
            $kernel_chksum2 = intval ($chksum_parts [1]);
            if ($kernel_chksum2 >= 38900/*$kernel_chksum == "3527659131 38900"*/) {
                // Dune Kernel SDK 2012_07_13 (новый, публичные прошивки 120531_2200_beta и новее)
                $sdk_version = "sdk20120713";
                $arescrypt_ko = dirname(__FILE__) . '/modules/' .
                                $kernel_release . '/' . $smp_family . '/' .
                                $sdk_version . '/arescrypt.ko';
            } else if ($kernel_chksum == "3322656547 36120") {
                // Dune Kernel SDK 2011_10_19 (предыдущий, публичные прошивки 120329_2012 и старше)
            } else if ($kernel_chksum == "915029021 38163") {
                // Dune Kernel SDK 2012_12_28 (Linux kernel:  2.6.32, Dune HD TV-102, Dune HD Connect)
            }
        }
        hd_print ("arescrypt_ko: $arescrypt_ko");

        // 4. insmod arescrypt.ko

        $cmd = sprintf ('insmod %s', escapeshellarg ($arescrypt_ko));
        hd_print ("cmd_insmod: $cmd");
        system ($cmd, $rc);
        if ($rc !== 0) {
            hd_print ("Error: insmod failed");
            throw_ares_error ();
        }

        // 5. start arescam daemon

        $cmd = sprintf (IptvportalConfig::cmd_daemon, escapeshellcmd (IptvportalPlugin::$arescam_exe),
                                           escapeshellarg (IptvportalConfig::auth_url),
                                           escapeshellarg (IptvportalConfig::daemon_pid_file));

        $terminal_params = IptvportalPlugin::get_terminal_params ();
        if ($terminal_params) {
            if (isset ($terminal_params->use_mcast_proxy) && $terminal_params->use_mcast_proxy &&
                isset ($terminal_params->mcast_proxy_url) && $terminal_params->mcast_proxy_url) {
                $mcast_proxy_url = $terminal_params->mcast_proxy_url;
                if (strpos ($mcast_proxy_url, '://') === false)
                    $mcast_proxy_url = 'http://' . $mcast_proxy_url;
                $cmd = $cmd . ' -y ' . $mcast_proxy_url;
            }
        }
        hd_print ("cmd_daemon: $cmd");
        system ($cmd, $rc);
        if ($rc !== 0) {
            hd_print("Error: daemon failed to start");
            throw_ares_error();
        }

        // 6. check PID file

        hd_print ("--> Checking PID-file '" . IptvportalConfig::daemon_pid_file . "'...");
        for ($i = 0; $i < 5; ++$i) {
            hd_silence_warnings ();
            $pid = file_get_contents (IptvportalConfig::daemon_pid_file);
            hd_restore_warnings ();
            if ($pid !== false) {
                hd_print ("--> Daemon started; pid: $pid");
                break;
            }
            hd_print ('--> No PID-file. Waiting...');
            usleep (100000);
        }
        if ($pid === false) // XXX: that might be actually an error...
            hd_print ("Warning: no PID-file within 0.5 sec after start.");
    }

    public static function api_call ($json_request) {
        $json_reply = null;
        for ($i=0; $i<3; ++$i) {
            try {
                // hd_print('--> json_request: "' . json_encode($json_request) . '"');
                $doc = HD::http_post_document (IptvportalConfig::api_url,
                                               json_encode ($json_request));
                //hd_print("JSON-reply: '$doc'");
                $json_reply = json_decode ($doc);
            } catch (Exception $e) {
                hd_print ("Error: failed to do HTTP-request.");
                if ($i == 2)
                    throw_server_unavailable_error ();
                usleep (100000 << $i);
                continue;
            }
            break;
        }
        if (is_null ($json_reply)) {
            hd_print ("Error: failed to decode API reply: '$doc'");
            throw_api_error ();
        }
        if (isset ($json_reply->error) && $json_reply->error) {
            hd_print ("Error: API returned error status ($doc)");
            throw_api_error ();
        }
        return $json_reply;
    }

    public static function get_terminal_params () {
        $json_reply = IptvportalPlugin::api_call (
            HD::make_json_rpc_request ('get_terminal_params', array (
                'macaddr' => str_replace (':', '-', HD::get_mac_addr ())
            ))
        );
        if (is_null ($json_reply) || is_null ($json_reply->result)) {
            hd_print ('Error: get_terminal_params');
            return false;
        }
        return $json_reply->result;
    }
}


?>
